# Copyright (c) 2018-2024 by xcube team and contributors
# Permissions are hereby granted under the terms of the MIT License:
# https://opensource.org/licenses/MIT.

from xcube.util.jsonschema import JsonObjectSchema

# Note: we could limit/enhance the default expression capabilities here
CONFIG_SCHEMA = JsonObjectSchema(additional_properties=True)
