# Copyright (c) 2018-2024 by xcube team and contributors
# Permissions are hereby granted under the terms of the MIT License:
# https://opensource.org/licenses/MIT.

from .routes import PATH_PARAM_DATASET_ID
from .routes import PATH_PARAM_VAR_NAME
from .routes import QUERY_PARAM_CMAP
from .routes import QUERY_PARAM_NORM
from .routes import QUERY_PARAM_CRS
from .routes import QUERY_PARAM_VMAX
from .routes import QUERY_PARAM_VMIN
from .routes import api
